#include <assert.h>
#include <stddef.h>
#include <stdlib.h>

#include "framebuffer.h"

void fb_draw_pixel(uint16_t *image, uint16_t x, uint16_t y, uint16_t color)
{
    assert(image != NULL);

    image[x + 240 * y] = color;
}

#ifndef _swap_int16_t
#define _swap_int16_t(a, b) { int16_t t = a; a = b; b = t; }
#endif

void fb_draw_line(uint16_t *image, int16_t x0, int16_t y0, int16_t x1, int16_t y1, uint16_t color)
{
    assert(image != NULL);

    int16_t steep = abs(y1 - y0) > abs(x1 - x0);
    if(steep) {
        _swap_int16_t(x0, y0);
        _swap_int16_t(x1, y1);
    }
    if(x0 > x1) {
        _swap_int16_t(x0, x1);
        _swap_int16_t(y0, y1);
    }

    int16_t dx, dy;
    dx = x1 - x0;
    dy = abs(y1 - y0);

    int16_t err = dx / 2;
    int16_t ystep;

    if(y0 < y1) {
        ystep = 1;
    } else {
        ystep = -1;
    }

    for(; x0 <= x1; x0++) {
        if(steep) {
            fb_draw_pixel(image, y0, x0, color);
        } else {
            fb_draw_pixel(image, x0, y0, color);
        }
        err -= dy;
        if (err < 0) {
            y0 += ystep;
            err += dx;
        }
    }
}

void fb_load_image(uint16_t *dst, uint16_t *src, uint32_t length)
{
    assert(NULL != dst);
    assert(NULL != src);

    for(uint32_t i = 0; i < length; i++) {
        dst[i] = src[i];
    }
}

void fb_set_image(uint16_t *dst, uint16_t value, uint32_t length)
{
    assert(NULL != dst);

    for (uint32_t i = 0; i < length; i++) {
        dst[i] = value;
    }
}

